﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Runtime.InteropServices;
using System.IO;

namespace Comparer
{
	public class Iconizer
	{
		#region Win32 declarations
		private const uint SHGFI_ICON = 0x100;
		private const uint SHGFI_LARGEICON = 0x0;
		private const uint SHGFI_SMALLICON = 0x1;

		[StructLayout( LayoutKind.Sequential )]
		public struct SHFILEINFO
		{
			public IntPtr hIcon;
			public IntPtr iIcon;
			public uint dwAttributes;
			[MarshalAs( UnmanagedType.ByValTStr, SizeConst = 260 )]
			public string szDisplayName;
			[MarshalAs( UnmanagedType.ByValTStr, SizeConst = 80 )]
			public string szTypeName;
		};

		[DllImport( "shell32.dll" )]
		public static extern IntPtr SHGetFileInfo( string pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, uint cbSizeFileInfo, uint uFlags );
		#endregion


		public static Bitmap ReturnIcon( string FileName )
		{
			SHFILEINFO shinfo = new SHFILEINFO();

			FileInfo info = new FileInfo( FileName );

			string ext = info.Extension;

			if (String.IsNullOrEmpty( ext ))
			{
				if ((info.Attributes & FileAttributes.Directory) != 0)
				{
					ext = "5EEB255733234c4dBECF9A128E896A1E"; // for directories
				}
				else
				{
					ext = "F9EB930C78D2477c80A51945D505E9C4"; // for files without extension
				}
			}
			else
			{
				if (ext.Equals( ".exe", StringComparison.InvariantCultureIgnoreCase ) || ext.Equals( ".lnk", StringComparison.InvariantCultureIgnoreCase ))
				{
					ext = info.Name;
				}
			}


			SHGetFileInfo( FileName, 0, ref shinfo, (uint) Marshal.SizeOf( shinfo ), SHGFI_ICON | SHGFI_LARGEICON );

			Icon ikona;

			try
			{
				ikona = Icon.FromHandle( shinfo.hIcon );
			}

			catch (ArgumentException ex)
			{
				throw new ArgumentException( String.Format( "File \"{0}\" doesn not exist!", FileName ), ex );
			}

			return ikona.ToBitmap();

		}

	}
}
